@@ -287,9 +287,11 @@ impl Allocator {
             if page != 0u64 {
                 let next = self.disc.mark_allocated(page)?;
                 fl.set(exp, next);
-                let mut pg = self.write_page_int(page)?;
-                pg.reset()?;
-                return Ok(pg);
+                {
+                    let mut cache = self.cache.lock()?;
+                    cache.remove(page);
+                }
+                return Ok(Page::new_alloc(page, exp));
             }
         }
         let page = self.disc.create_page(exp)?;
@@ -404,7 +406,7 @@ impl Allocator {
         Ok(order)
     }
 
-    pub fn write_root_page(&self, mut root: Page, buffer: &mut [u8], version: u8, last_flush: u8) -> PERes<u8> {
+    fn write_root_page(&self, mut root: Page, buffer: &mut [u8], version: u8, last_flush: u8) -> PERes<u8> {
         let order = write_root_page(&mut root, buffer, version, last_flush)?;
         self.flush_page(root)?;
         Ok(order)
