@@ -33,7 +33,7 @@ fn read_id(s: &str) -> Result<u64, InvalidPersyId> {
         .0)
 }
 
-#[derive(PartialEq, Eq, PartialOrd, Ord, Hash, Clone, Debug, Default)]
+#[derive(PartialEq, Eq, PartialOrd, Ord, Hash, Clone, Debug, Default, Copy)]
 pub struct RecRef {
     pub page: u64,
     pub pos: u32,
@@ -79,7 +79,7 @@ impl std::str::FromStr for RecRef {
 }
 
 /// Identifier of a persistent record, can be used for read, update or delete the record
-#[derive(PartialEq, Eq, PartialOrd, Ord, Hash, Clone, Debug)]
+#[derive(PartialEq, Eq, PartialOrd, Ord, Hash, Clone, Debug, Copy)]
 pub struct PersyId(pub(crate) RecRef);
 
 impl fmt::Display for PersyId {
