@@ -2,13 +2,15 @@ pub mod nodes;
 use crate::{
     error::{PERes, PIRes},
     index::{
-        config::{IndexOrd, IndexType, ValueMode},
+        config::{IndexOrd, ValueMode},
         keeper::{IndexKeeper, IndexModify},
         tree::nodes::{Leaf, Node, NodeRef, Nodes, PageIter, PageIterBack, Value},
     },
 };
 use std::{cmp::Ordering, ops::Bound, rc::Rc};
 
+use super::config::IndexTypeInternal;
+
 pub enum ValueChange<V> {
     Add(V),
     Remove(Option<V>),
@@ -19,7 +21,7 @@ pub struct KeyChanges<K, V> {
     changes: Vec<ValueChange<V>>,
 }
 
-impl<K: IndexType, V: IndexType> KeyChanges<K, V> {
+impl<K: IndexTypeInternal, V: IndexTypeInternal> KeyChanges<K, V> {
     #[allow(dead_code)]
     pub fn single_add(k: K, v: V) -> KeyChanges<K, V> {
         KeyChanges {
@@ -68,6 +70,24 @@ pub trait IndexApply<K, V>: Index<K, V> {
     fn apply(&mut self, adds: &[KeyChanges<K, V>]) -> PIRes<()>;
 }
 
+struct Updated<K> {
+    path: Vec<PosRef<K>>,
+}
+impl<K> Updated<K> {
+    fn new() -> Self {
+        Self { path: Vec::new() }
+    }
+    fn push(&mut self, p: PosRef<K>) {
+        self.path.push(p)
+    }
+    fn clear(&mut self) {
+        self.path.clear();
+    }
+    fn last(&self) -> Option<&PosRef<K>> {
+        self.path.last()
+    }
+}
+
 #[derive(PartialEq, Clone, Debug)]
 pub struct PosRef<K> {
     pub k: K,
@@ -90,10 +110,19 @@ impl<K: Clone> PosRef<K> {
 }
 
 struct ParentNodeChanged<K> {
-    path: Vec<PosRef<K>>,
+    path: Updated<K>,
     children: Vec<PosRef<K>>,
 }
 
+impl<K> ParentNodeChanged<K> {
+    fn node_id(&self) -> NodeRef {
+        self.path.last().unwrap().node_ref
+    }
+    fn is_root(&self) -> bool {
+        self.path.path.is_empty()
+    }
+}
+
 fn split_link_save<K: IndexOrd + Clone, V, I: IndexModify<K, V>>(
     index: &mut I,
     mut prev_id: NodeRef,
@@ -114,7 +143,7 @@ fn split_link_save<K: IndexOrd + Clone, V, I: IndexModify<K, V>>(
             let (sibling_node, version) = index.load_modify(id)?.unwrap();
             if let Node::Leaf(sibling) = index.owned(id, sibling_node) {
                 index.update(&prev_id, Node::Leaf(prev_leaf), prev_version)?;
-                prev_id = id.clone();
+                prev_id = *id;
                 prev_leaf = sibling;
                 prev_version = version;
             }
@@ -126,13 +155,13 @@ fn split_link_save<K: IndexOrd + Clone, V, I: IndexModify<K, V>>(
     Ok(ids)
 }
 
-fn group_by_parent<K>(updates: Vec<Vec<PosRef<K>>>) -> Vec<ParentNodeChanged<K>> {
+fn group_by_parent<K>(updates: Vec<Updated<K>>) -> Vec<ParentNodeChanged<K>> {
     let mut parent_updates = Vec::new();
     let mut parent_node: Option<NodeRef> = None;
     let mut new_update: Option<ParentNodeChanged<K>> = None;
     for mut update in updates {
-        if let Some(last) = update.pop() {
-            if parent_node == update.last().map(|x| x.node_ref.clone()) {
+        if let Some(last) = update.path.pop() {
+            if parent_node == update.path.last().map(|x| x.node_ref) {
                 if let Some(p) = &mut new_update {
                     p.children.push(last);
                 }
@@ -140,7 +169,7 @@ fn group_by_parent<K>(updates: Vec<Vec<PosRef<K>>>) -> Vec<ParentNodeChanged<K>>
                 if let Some(p) = new_update {
                     parent_updates.push(p);
                 }
-                parent_node = update.last().map(|x| x.node_ref.clone());
+                parent_node = update.path.last().map(|x| x.node_ref);
                 let children = vec![last];
                 new_update = Some(ParentNodeChanged { path: update, children });
             }
@@ -153,18 +182,18 @@ fn group_by_parent<K>(updates: Vec<Vec<PosRef<K>>>) -> Vec<ParentNodeChanged<K>>
 }
 fn lock_parents<K: IndexOrd + Clone, V, I: IndexModify<K, V>>(
     index: &mut I,
-    mut updates: Vec<Vec<PosRef<K>>>,
-) -> PIRes<Vec<Vec<PosRef<K>>>> {
+    mut updates: Vec<Updated<K>>,
+) -> PIRes<Vec<Updated<K>>> {
     for update in &mut updates {
         loop {
             debug_assert!(
-                update.len() >= 2,
+                update.path.len() >= 2,
                 "never lock a path that shorter than 2 because that would be the root",
             );
             let locked = {
                 let update = &update;
-                let len = update.len();
-                let rec_ref = &update[len - 2];
+                let len = update.path.len();
+                let rec_ref = &update.path[len - 2];
                 if let Some((node, version)) = index.load_modify(&rec_ref.node_ref)? {
                     lock_logic(index, rec_ref, node, version)?.is_some()
                 } else {
@@ -174,10 +203,10 @@ fn lock_parents<K: IndexOrd + Clone, V, I: IndexModify<K, V>>(
             if locked {
                 break;
             } else {
-                let last = update.last().unwrap().clone();
+                let last = update.path.last().unwrap().clone();
                 if let Some(ref node) = index.get_root_refresh()? {
                     let mut path = Vec::new();
-                    let mut cur_node = PosRef::new(&last.k, 0, node.clone(), None);
+                    let mut cur_node = PosRef::new(&last.k, 0, *node, None);
                     while cur_node.node_ref != last.node_ref {
                         let mut restart = true;
                         if let Some((node_modify, version)) = index.load_modify(&cur_node.node_ref)? {
@@ -192,7 +221,7 @@ fn lock_parents<K: IndexOrd + Clone, V, I: IndexModify<K, V>>(
                         }
                         if restart {
                             if let Some(ref node) = index.get_root_refresh()? {
-                                cur_node = PosRef::new(&last.k, 0, node.clone(), None);
+                                cur_node = PosRef::new(&last.k, 0, *node, None);
                                 path.clear();
                             } else {
                                 panic!("restart node finding but not root present");
@@ -200,7 +229,7 @@ fn lock_parents<K: IndexOrd + Clone, V, I: IndexModify<K, V>>(
                         }
                     }
                     path.push(last);
-                    *update = path;
+                    update.path = path;
                 }
             }
         }
@@ -290,7 +319,110 @@ fn lock_logic<K: IndexOrd + Clone, V, I: IndexModify<K, V>>(
     }
 }
 
-impl<K: IndexType, V: IndexType, T> IndexApply<K, V> for T
+fn find_and_change_leaf<K: IndexTypeInternal, V: IndexTypeInternal, I: IndexModify<K, V>>(
+    add: &KeyChanges<K, V>,
+    updates: &mut Vec<Updated<K>>,
+    index: &mut I,
+    mut prev_leaf_id: Option<NodeRef>,
+) -> PIRes<Option<NodeRef>> {
+    let mut next: Option<PosRef<K>> = None;
+    let mut path = Updated::new();
+    loop {
+        next = if let Some(cur_node) = &mut next {
+            let mut new_next = None;
+            if let Some((node_modify, version)) = index.load_modify(&cur_node.node_ref)? {
+                if let Node::Node(n) = &*node_modify {
+                    cur_node.version = version;
+                    path.push(cur_node.clone());
+                    new_next = n.find_write(&add.k);
+                } else if let Node::Leaf(ref _none) = &*node_modify {
+                    cur_node.version = version;
+                    if let Some((leaf_modify, version)) = lock_logic(index, cur_node, node_modify, version)? {
+                        path.push(cur_node.clone());
+                        if let Node::Leaf(mut leaf) = index.owned(&cur_node.node_ref, leaf_modify) {
+                            let node_ref = &cur_node.node_ref;
+                            if add.apply(&mut leaf, index.value_mode(), index.index_name())? {
+                                index.update(node_ref, Node::Leaf(leaf), version)?;
+                                if Some(*node_ref) != prev_leaf_id {
+                                    updates.push(path);
+                                }
+                                prev_leaf_id = Some(*node_ref)
+                            }
+                            break;
+                        }
+                    }
+                }
+            }
+            new_next
+        } else if let Some(r) = index.get_root_refresh()? {
+            path.clear();
+            Some(PosRef::new(&add.k, 0, r, None))
+        } else {
+            index.lock_config()?;
+            if let Some(r) = index.get_root_refresh()? {
+                path.clear();
+                Some(PosRef::new(&add.k, 0, r, None))
+            } else {
+                let mut leaf = Leaf::new();
+                add.apply(&mut leaf, index.value_mode(), index.index_name())?;
+                let leaf_ref = index.insert(Node::Leaf(leaf))?;
+                index.set_root(Some(leaf_ref))?;
+                break;
+            }
+        }
+    }
+    Ok(prev_leaf_id)
+}
+fn merge_and_split_children<K: IndexOrd + Clone, V: Clone, I: IndexModify<K, V>>(
+    mut n: Nodes<K>,
+    children: Vec<PosRef<K>>,
+    index: &mut I,
+) -> PIRes<Option<Nodes<K>>> {
+    let mut save = false;
+    let mut changed_flags = vec![false; n.len()];
+    for ch in children {
+        let pos = n.find(&ch.k);
+        changed_flags[pos.pos] = true;
+        if n.len() > 1 {
+            // It's locked, should not have miss read unwrap.
+            let cur_id = pos.node_ref;
+            let (cur, version) = index.load_modify(&cur_id)?.unwrap();
+            if cur.len() < index.bottom_limit() {
+                let mut new_pos = pos.pos;
+                let to_modify = index.owned(&cur_id, cur);
+                if merge_and_save(index, &mut n, pos.pos, to_modify, version)? {
+                    new_pos += 1;
+                }
+                changed_flags.remove(new_pos);
+                changed_flags[new_pos - 1] = true;
+                save = true;
+            }
+        }
+    }
+    for pos in 0..n.len() {
+        if changed_flags[pos] {
+            // It's locked, should not have miss read unwrap.
+            let cur_id = n.get(pos);
+            let (cur, cur_version) = index.load_modify(&cur_id)?.unwrap();
+            if cur.len() > index.top_limit() {
+                let to_modify = index.owned(&cur_id, cur);
+                let mut ids = split_link_save(index, n.get(pos), to_modify, cur_version)?;
+                for _ in 0..ids.len() {
+                    changed_flags.insert(pos, false);
+                }
+                n.insert_after(pos, &mut ids);
+                save = true;
+            }
+        }
+    }
+    if save {
+        Ok(Some(n))
+    } else {
+        Ok(None)
+    }
+}
+
+impl<K: IndexTypeInternal, V: IndexTypeInternal, T> IndexApply<K, V> for T
 where
     T: IndexModify<K, V>,
     T: Index<K, V>,
@@ -299,108 +431,21 @@ where
         let mut updates = Vec::new();
         let mut prev_leaf_id = None;
         for add in adds {
-            let mut next: Option<PosRef<K>> = None;
-            let mut path = Vec::new();
-            loop {
-                next = if let Some(cur_node) = &mut next {
-                    let mut new_next = None;
-                    if let Some((node_modify, version)) = self.load_modify(&cur_node.node_ref)? {
-                        if let Node::Node(n) = &*node_modify {
-                            cur_node.version = version;
-                            path.push(cur_node.clone());
-                            new_next = n.find_write(&add.k);
-                        } else if let Node::Leaf(ref _none) = &*node_modify {
-                            cur_node.version = version;
-                            if let Some((leaf_modify, version)) = lock_logic(self, cur_node, node_modify, version)? {
-                                path.push(cur_node.clone());
-                                if let Node::Leaf(mut leaf) = self.owned(&cur_node.node_ref, leaf_modify) {
-                                    let node_ref = &cur_node.node_ref;
-                                    if add.apply(&mut leaf, self.value_mode(), self.index_name())? {
-                                        self.update(node_ref, Node::Leaf(leaf), version)?;
-                                        if Some(node_ref.clone()) != prev_leaf_id {
-                                            updates.push(path);
-                                        }
-                                        prev_leaf_id = Some(node_ref.clone())
-                                    }
-                                    break;
-                                }
-                            }
-                        }
-                    }
-                    new_next
-                } else if let Some(r) = self.get_root_refresh()? {
-                    path.clear();
-                    Some(PosRef::new(&add.k, 0, r.clone(), None))
-                } else {
-                    self.lock_config()?;
-                    if let Some(r) = self.get_root_refresh()? {
-                        path.clear();
-                        Some(PosRef::new(&add.k, 0, r.clone(), None))
-                    } else {
-                        let mut leaf = Leaf::new();
-                        add.apply(&mut leaf, self.value_mode(), self.index_name())?;
-                        let leaf_ref = self.insert(Node::Leaf(leaf))?;
-                        self.set_root(Some(leaf_ref))?;
-                        break;
-                    }
-                }
-            }
+            prev_leaf_id = find_and_change_leaf(add, &mut updates, self, prev_leaf_id)?;
         }
-        while updates.len() > 1 || (updates.len() == 1 && updates[0].len() > 1) {
+        while updates.len() > 1 || (updates.len() == 1 && updates[0].path.len() > 1) {
             updates = lock_parents(self, updates)?;
             let parent_updates = group_by_parent(updates);
             updates = Vec::new();
             for update in parent_updates {
-                let parent_id = update.path.last().unwrap().node_ref.clone();
+                let parent_id = update.node_id();
                 // It's locked, should not have miss read unwrap.
                 let (read_node, n_version) = self.load_modify(&parent_id)?.unwrap();
-                if let Node::Node(mut n) = self.owned(&parent_id, read_node) {
-                    let mut save = false;
-                    let mut flags = vec![false; n.len()];
-                    for ch in update.children {
-                        let pos = n.find(&ch.k);
-                        flags[pos.pos] = true;
-                    }
-                    let mut i = 0;
-                    while i < n.len() {
-                        let pos = i;
-                        i += 1;
-                        if flags[pos] && n.len() > 1 {
-                            // It's locked, should not have miss read unwrap.
-                            let cur_id = n.get(pos);
-                            let (cur, version) = self.load_modify(&cur_id)?.unwrap();
-                            if cur.len() < self.bottom_limit() {
-                                let mut new_pos = pos;
-                                let to_modify = self.owned(&cur_id, cur);
-                                if merge_and_save(self, &mut n, pos, to_modify, version)? {
-                                    new_pos += 1;
-                                }
-                                flags.remove(new_pos);
-                                flags[new_pos - 1] = true;
-                                i = new_pos - 1;
-                                save = true;
-                            }
-                        }
-                    }
-                    for pos in 0..n.len() {
-                        if flags[pos] {
-                            // It's locked, should not have miss read unwrap.
-                            let cur_id = n.get(pos);
-                            let (cur, cur_version) = self.load_modify(&cur_id)?.unwrap();
-                            if cur.len() > self.top_limit() {
-                                let to_modify = self.owned(&cur_id, cur);
-                                let mut ids = split_link_save(self, n.get(pos), to_modify, cur_version)?;
-                                for _ in 0..ids.len() {
-                                    flags.insert(pos, false);
-                                }
-                                n.insert_after(pos, &mut ids);
-                                save = true;
-                            }
-                        }
-                    }
-                    if save {
+                if let Node::Node(n) = self.owned(&parent_id, read_node) {
+                    let is_root = update.is_root();
+                    if let Some(n) = merge_and_split_children(n, update.children, self)? {
                         self.update(&parent_id, Node::Node(n), n_version)?;
-                        if !update.path.is_empty() {
+                        if !is_root {
                             updates.push(update.path);
                         } else {
                             break;
@@ -409,13 +454,13 @@ where
                 }
             }
         }
-        if updates.len() == 1 && updates[0].len() == 1 {
+        if updates.len() == 1 && updates[0].path.len() == 1 {
             self.lock_config()?;
             while let Some(r) = self.get_root_refresh()? {
                 if let Some((n, n_version)) = self.load_modify(&r)? {
                     if n.len() > self.top_limit() {
                         let to_modify = self.owned(&r, n);
-                        let ids = split_link_save(self, r.clone(), to_modify, n_version)?;
+                        let ids = split_link_save(self, r, to_modify, n_version)?;
                         let node = Node::Node(Nodes::new_from_split(r, &ids));
                         let cur_id = self.insert(node)?;
                         self.set_root(Some(cur_id))?;
@@ -438,7 +483,7 @@ where
     }
 }
 
-impl<K: IndexType, V: IndexType, T> Index<K, V> for T
+impl<K: IndexTypeInternal, V: IndexTypeInternal, T> Index<K, V> for T
 where
     T: IndexKeeper<K, V>,
 {
@@ -463,7 +508,7 @@ where
     fn iter_from(&mut self, first: Bound<&K>) -> PERes<PageIter<K, V>> {
         let mut path = Vec::new();
         let mut iter = if let Some(mut cur_node) = self.get_root()? {
-            path.push((0, cur_node.clone()));
+            path.push((0, cur_node));
             loop {
                 match self.load(&cur_node)? {
                     Node::Node(n) => {
@@ -478,7 +523,7 @@ where
                             }
                             Bound::Unbounded => (0, n.get(0)),
                         };
-                        cur_node = value.1.clone();
+                        cur_node = value.1;
                         path.push(value);
                     }
                     Node::Leaf(leaf) => {
@@ -534,7 +579,7 @@ where
     fn back_iter_from(&mut self, last: Bound<&K>) -> PERes<PageIterBack<K, V>> {
         let mut path = Vec::new();
         let mut iter = if let Some(mut cur_node) = self.get_root()? {
-            path.push((0, cur_node.clone()));
+            path.push((0, cur_node));
             loop {
                 match self.load(&cur_node)? {
                     Node::Node(n) => {
@@ -552,7 +597,7 @@ where
                                 (pos, n.get(pos))
                             }
                         };
-                        cur_node = value.1.clone();
+                        cur_node = value.1;
                         path.push(value);
                     }
                     Node::Leaf(leaf) => {
