@@ -87,7 +87,8 @@ pub use crate::{
     },
     id::{IndexId, PersyId, SegmentId, ToIndexId, ToSegmentId},
     index::{
-        config::{ByteVec, IndexType, IndexTypeId, ValueMode},
+        bytevec::ByteVec,
+        config::{IndexType, IndexTypeId, ValueMode},
         iter::{IndexIter, TxIndexIter},
         value_iter::ValueIter,
     },
@@ -373,7 +374,8 @@ impl Persy {
         Ok(self.persy_impl.solve_segment_id(segment)?)
     }
 
-    /// Resolves the index to a IndexId
+    /// Resolves the index to a IndexId,
+    /// this has no public use as today, may be used in future.
     ///
     /// # Example
     ///
@@ -385,7 +387,7 @@ impl Persy {
     /// tx.create_index::<u8,u8>("my_new_index", ValueMode::Cluster)?;
     /// let prepared = tx.prepare()?;
     /// prepared.commit()?;
-    /// let segment_id = persy.solve_index_id("my_new_index")?;
+    /// let index_id = persy.solve_index_id("my_new_index")?;
     /// # Ok(())
     /// # }
     /// ```
@@ -501,7 +503,10 @@ impl Persy {
         let index_id = self
             .solve_index_id(index_name)
             .map_err(|e| PE::PE(IndexOpsError::from(e.error())))?;
-        Ok(ValueIter::from(self.persy_impl.get::<K, V>(index_id, k)?))
+        Ok(ValueIter::from(
+            self.persy_impl
+                .get::<K::Wrapper, V::Wrapper>(index_id, &k.clone().wrap())?,
+        ))
     }
 
     /// Get one value or none from a key.
@@ -562,7 +567,8 @@ impl Persy {
         let index_id = self
             .solve_index_id(index_name)
             .map_err(|e| PE::PE(IndexOpsError::from(e.error())))?;
-        let (_, raw) = self.persy_impl.range(index_id, range)?;
+        let rr = PersyImpl::map_index_range_bounds(range);
+        let (_, raw) = self.persy_impl.range(index_id, rr)?;
         Ok(IndexIter::new(raw, self.persy_impl.clone()))
     }
 
