@@ -21,7 +21,7 @@ use crate::{
     },
     id::{IndexId, PersyId, RecRef, SegmentId, ToIndexId, ToSegmentId},
     index::{
-        config::{IndexType, IndexTypeId, Indexes, ValueMode, INDEX_DATA_PREFIX, INDEX_META_PREFIX},
+        config::{IndexType, IndexTypeId, IndexTypeInternal, Indexes, ValueMode, INDEX_DATA_PREFIX, INDEX_META_PREFIX},
         keeper::{IndexKeeper, IndexRawIter},
         tree::{
             nodes::{PageIter, PageIterBack, Value},
@@ -29,8 +29,8 @@ use crate::{
         },
     },
     io::{
-        InfallibleRead, InfallibleReadFormat, InfallibleReadVarInt, InfallibleWrite, InfallibleWriteFormat,
-        InfallibleWriteVarInt,
+        ArcSliceRead, InfallibleRead, InfallibleReadFormat, InfallibleReadVarInt, InfallibleWrite,
+        InfallibleWriteFormat, InfallibleWriteVarInt,
     },
     journal::{Journal, JournalId},
     record_scanner::{SegmentRawIter, SegmentSnapshotRawIter, TxSegmentRawIter},
@@ -190,7 +190,6 @@ impl PersyImpl {
             root.write_u64(allocator_page);
             root.write_all(PERSY_TAG_BYTES);
             allocator.flush_page(root)?;
-            // TODO: check this never go over the first page
         }
         allocator.disc_sync()?;
         Ok(allocator.release())
@@ -275,29 +274,35 @@ impl PersyImpl {
         let allocator = &self.allocator;
         for id in recover.order {
             if let Some((status, mut tx, choosed)) = recover.transactions.remove(&id) {
-                if status == RecoverStatus::PrepareCommit {
-                    if choosed == Some(CommitStatus::Commit) || choosed.is_none() {
-                        let prepared_result = tx.recover_prepare(self);
-                        if let Ok(prepared) = prepared_result {
-                            tx.recover_commit(self, prepared)?;
-                            self.journal.finished_to_clean(&[id])?;
+                match status {
+                    RecoverStatus::PrepareCommit => match choosed {
+                        Some(CommitStatus::Commit) | None => {
+                            let prepared_result = tx.recover_prepare(self);
+                            if let Ok(prepared) = prepared_result {
+                                tx.recover_commit(self, prepared)?;
+                                self.journal.finished_to_clean(&[id])?;
+                            }
                         }
-                    } else {
+                        Some(CommitStatus::Rollback) => {
+                            tx.recover_rollback(self)?;
+                        }
+                    },
+                    RecoverStatus::Commit => {
+                        tx.recover_cleanup(self)?;
+                        self.journal.finished_to_clean(&[id])?;
+                    }
+                    RecoverStatus::Cleanup => {
+                        self.journal.cleaned_to_trim(&[id])?;
+                    }
+                    RecoverStatus::Started | RecoverStatus::Rollback => {
                         tx.recover_rollback(self)?;
                     }
-                } else if status == RecoverStatus::Commit {
-                    tx.recover_cleanup(self)?;
-                    self.journal.finished_to_clean(&[id])?;
-                } else if status == RecoverStatus::Cleanup {
-                    self.journal.cleaned_to_trim(&[id])?;
                 }
             }
         }
         for (_, (_, tx, _)) in recover.transactions.iter_mut() {
             tx.recover_rollback(self)?;
         }
-        self.address.recompute_last_pages()?;
-        self.address.flush_segments()?;
         allocator.disc_sync()?;
         self.journal.clear_in_queque()?;
         allocator.trim_free_at_end()?;
@@ -510,14 +515,14 @@ impl PersyImpl {
     }
 
     fn read_page(&self, match_id: &RecRef, page: u64) -> PERes<Option<Vec<u8>>> {
-        self.read_page_fn(match_id, page, |x| Vec::from(x))
+        self.read_page_fn(match_id, page, |x| x.to_vec())
     }
 
-    fn read_page_fn<T, F: Fn(&[u8]) -> T>(&self, match_id: &RecRef, page: u64, f: F) -> PERes<Option<T>> {
+    fn read_page_fn<T, F: Fn(ArcSliceRead) -> T>(&self, match_id: &RecRef, page: u64, f: F) -> PERes<Option<T>> {
         if let Some(mut pg) = self.allocator.load_page_not_free(page)? {
             let (len, id) = PersyImpl::read_record_metadata(&mut pg);
             if id.page == match_id.page && id.pos == match_id.pos {
-                Ok(Some(f(pg.slice(len as usize))))
+                Ok(Some(f(pg.arc_slice(len as usize))))
             } else {
                 Ok(None)
             }
@@ -531,7 +536,7 @@ impl PersyImpl {
         tx: &TransactionImpl,
         segment_id: SegmentId,
         id: &RecRef,
-        f: fn(&[u8]) -> T,
+        f: fn(ArcSliceRead) -> T,
     ) -> Result<Option<(T, u16)>, ReadError> {
         let mut retry_count = 0;
         loop {
@@ -558,7 +563,7 @@ impl PersyImpl {
         segment_id: SegmentId,
         id: &RecRef,
     ) -> Result<Option<(Vec<u8>, u16)>, ReadError> {
-        self.read_tx_internal_fn(tx, segment_id, id, |x| Vec::from(x))
+        self.read_tx_internal_fn(tx, segment_id, id, |x| x.to_vec())
     }
 
     pub fn read_tx(
@@ -615,7 +620,7 @@ impl PersyImpl {
         rec_ref: &RecRef,
         snapshot: SnapshotId,
     ) -> Result<Option<Vec<u8>>, ReadError> {
-        self.read_snap_fn(segment, rec_ref, snapshot, |x| Vec::from(x))
+        self.read_snap_fn(segment, rec_ref, snapshot, |x| x.to_vec())
     }
 
     pub fn read_snap_fn<T>(
@@ -623,7 +628,7 @@ impl PersyImpl {
         segment: SegmentId,
         rec_ref: &RecRef,
         snapshot: SnapshotId,
-        f: fn(&[u8]) -> T,
+        f: fn(ArcSliceRead) -> T,
     ) -> Result<Option<T>, ReadError> {
         let segment_id = segment;
         if let Some(rec_vers) = self.snapshots.read(snapshot, rec_ref)? {
@@ -705,7 +710,7 @@ impl PersyImpl {
             pg.write_all(rec);
             Ok(allocator.flush_page(pg)?)
         } else {
-            Err(UpdateError::RecordNotFound(PersyId(rec_ref.clone())))
+            Err(UpdateError::RecordNotFound(PersyId(*rec_ref)))
         }
     }
 
@@ -713,7 +718,7 @@ impl PersyImpl {
         if let Some((_, version, seg)) = self.read_ref_segment(tx, segment, rec_ref)? {
             Ok(tx.add_delete(&self.journal, seg, rec_ref, version)?)
         } else {
-            Err(DeleteError::RecordNotFound(PersyId(rec_ref.clone())))
+            Err(DeleteError::RecordNotFound(PersyId(*rec_ref)))
         }
     }
 
@@ -752,7 +757,7 @@ impl PersyImpl {
         K: IndexType,
         V: IndexType,
     {
-        Indexes::create_index::<K, V>(self, tx, index_name, 32, 128, value_mode)
+        Indexes::create_index::<K::Wrapper, V::Wrapper>(self, tx, index_name, 32, 128, value_mode)
     }
 
     pub fn drop_index(&self, tx: &mut TransactionImpl, index_name: &str) -> Result<(), DropIndexError> {
@@ -761,8 +766,8 @@ impl PersyImpl {
 
     pub fn put<K, V>(&self, tx: &mut TransactionImpl, index_id: IndexId, k: K, v: V) -> Result<(), IndexOpsError>
     where
-        K: IndexType,
-        V: IndexType,
+        K: IndexTypeInternal,
+        V: IndexTypeInternal,
     {
         Indexes::check_index::<K, V>(self, tx, &index_id)?;
         tx.add_put(index_id, k, v);
@@ -777,8 +782,8 @@ impl PersyImpl {
         v: Option<V>,
     ) -> Result<(), IndexOpsError>
     where
-        K: IndexType,
-        V: IndexType,
+        K: IndexTypeInternal,
+        V: IndexTypeInternal,
     {
         Indexes::check_index::<K, V>(self, tx, &index_id)?;
         tx.add_remove(index_id, k, v);
@@ -792,22 +797,26 @@ impl PersyImpl {
         k: &K,
     ) -> Result<Option<Value<V>>, IndexChangeError>
     where
-        K: IndexType,
-        V: IndexType,
+        K: IndexTypeInternal,
+        V: IndexTypeInternal,
     {
-        let (result, vm) = {
+        let (result, vm, name) = {
             let mut ik = Indexes::get_index_keeper_tx::<K, V>(self, tx, &index_id)?;
             self.indexes.read_lock(index_id.clone())?;
-            (ik.get(k)?, IndexKeeper::<K, V>::value_mode(&ik))
+            (
+                ik.get(k)?,
+                IndexKeeper::<K, V>::value_mode(&ik),
+                ik.index_name().clone(),
+            )
         };
         self.indexes.read_unlock(index_id.clone())?;
-        tx.apply_changes::<K, V>(vm, index_id, k, result)
+        tx.apply_changes::<K, V>(vm, index_id, &name, k, result)
     }
 
     pub fn get<K, V>(&self, index_id: IndexId, k: &K) -> Result<Option<Value<V>>, IndexOpsError>
     where
-        K: IndexType,
-        V: IndexType,
+        K: IndexTypeInternal,
+        V: IndexTypeInternal,
     {
         let read_snapshot = self.snapshots.read_snapshot()?;
         let r = self.get_snapshot(index_id, read_snapshot, k);
@@ -822,8 +831,8 @@ impl PersyImpl {
         k: &K,
     ) -> Result<Option<Value<V>>, IndexOpsError>
     where
-        K: IndexType,
-        V: IndexType,
+        K: IndexTypeInternal,
+        V: IndexTypeInternal,
     {
         Ok(Indexes::get_index_keeper::<K, V>(self, snapshot, &index_id)?.get(k)?)
     }
@@ -835,8 +844,8 @@ impl PersyImpl {
         next: Bound<&K>,
     ) -> Result<PageIter<K, V>, IndexOpsError>
     where
-        K: IndexType,
-        V: IndexType,
+        K: IndexTypeInternal,
+        V: IndexTypeInternal,
     {
         Ok(Indexes::get_index_keeper::<K, V>(self, read_snapshot, index_id)?.iter_from(next)?)
     }
@@ -848,16 +857,32 @@ impl PersyImpl {
         next: Bound<&K>,
     ) -> Result<PageIterBack<K, V>, IndexOpsError>
     where
-        K: IndexType,
-        V: IndexType,
+        K: IndexTypeInternal,
+        V: IndexTypeInternal,
     {
         Ok(Indexes::get_index_keeper::<K, V>(self, read_snapshot, index_id)?.back_iter_from(next)?)
     }
 
+    pub(crate) fn map_index_range_bounds<R: RangeBounds<X>, X: IndexType>(
+        r: R,
+    ) -> (Bound<X::Wrapper>, Bound<X::Wrapper>) {
+        (
+            match r.start_bound() {
+                Bound::Included(x) => Bound::Included(x.clone().wrap()),
+                Bound::Excluded(e) => Bound::Excluded(e.clone().wrap()),
+                Bound::Unbounded => Bound::Unbounded,
+            },
+            match r.end_bound() {
+                Bound::Included(x) => Bound::Included(x.clone().wrap()),
+                Bound::Excluded(e) => Bound::Excluded(e.clone().wrap()),
+                Bound::Unbounded => Bound::Unbounded,
+            },
+        )
+    }
     pub fn range<K, V, R>(&self, index_id: IndexId, range: R) -> Result<(ValueMode, IndexRawIter<K, V>), IndexOpsError>
     where
-        K: IndexType,
-        V: IndexType,
+        K: IndexTypeInternal,
+        V: IndexTypeInternal,
         R: RangeBounds<K>,
     {
         let read_snapshot = self.snapshots.read_snapshot()?;
@@ -872,8 +897,8 @@ impl PersyImpl {
         release_snapshot: bool,
     ) -> Result<(ValueMode, IndexRawIter<K, V>), IndexOpsError>
     where
-        K: IndexType,
-        V: IndexType,
+        K: IndexTypeInternal,
+        V: IndexTypeInternal,
         R: RangeBounds<K>,
     {
         let mut ik = Indexes::get_index_keeper::<K, V>(self, snapshot, &index_id)?;
@@ -892,8 +917,8 @@ impl PersyImpl {
         range: R,
     ) -> Result<(ValueMode, IndexRawIter<K, V>), IndexOpsError>
     where
-        K: IndexType,
-        V: IndexType,
+        K: IndexTypeInternal,
+        V: IndexTypeInternal,
         R: RangeBounds<K>,
     {
         let mut ik = Indexes::get_index_keeper_tx::<K, V>(self, tx, &index_id)?;
