@@ -1,12 +1,12 @@
 mod helpers;
 
-use helpers::{create_and_drop, CountDown};
-use persy::{Config, OpenOptions, Persy, PersyError};
+use helpers::{create_and_drop, create_and_drop_with_config, CountDown};
+use persy::{Config, OpenError, OpenOptions, Persy, ValueMode};
 use std::fs;
 use std::sync::atomic::{AtomicBool, Ordering::Relaxed};
 use std::sync::Arc;
 use std::thread;
-use tempfile::{tempfile, Builder};
+use tempfile::{tempfile, Builder, NamedTempFile};
 
 #[test]
 fn openoptions_truncate_locked() {
@@ -22,7 +22,8 @@ fn openoptions_truncate_locked() {
                 tx.insert("seg", &data[..])?;
 
                 let prepared = tx.prepare()?;
-                prepared.commit()
+                prepared.commit()?;
+                Ok(())
             })
             .open("./target/truncate-locked.persy")
             .unwrap();
@@ -39,8 +40,8 @@ fn openoptions_truncate_locked() {
         .unwrap_err();
 
     // it must be unsuccessful
-    match error {
-        PersyError::Io(_) => (),
+    match &*error {
+        OpenError::AlreadyInUse(_) => (),
         otherwise => panic!("error is {:?} and not an Io error", otherwise),
     }
 
@@ -83,8 +84,8 @@ fn openoptions_create_new() {
         .map(drop)
         .unwrap_err();
 
-    match error {
-        PersyError::Io(_) => (),
+    match &*error {
+        OpenError::AlreadyExists => (),
         otherwise => panic!("error is {:?} and not an Io error", otherwise),
     }
 
@@ -456,6 +457,68 @@ pub fn test_multiple_open_tx_close() {
     }
 }
 
+#[test]
+pub fn test_multiple_open_close_restore_tx() {
+    let file = Builder::new()
+        .prefix("multiple_open_restore_close_tx")
+        .suffix(".persy")
+        .tempfile()
+        .expect("expect temp file creation");
+    Persy::create_from_file(file.reopen().unwrap()).unwrap();
+    {
+        let persy = Persy::open_from_file(file.reopen().unwrap(), Config::new()).unwrap();
+        let mut tx = persy.begin().expect("error on transaction begin");
+        tx.create_segment("def").expect("error on segment creation");
+        let fin = tx.prepare().expect("error on commit prepare");
+        fin.commit().expect("error on commit");
+    }
+    for ite in 1..20 {
+        let recover = OpenOptions::new().recover_file(file.reopen().unwrap()).unwrap();
+        assert!(recover.list_transactions().len() <= 10);
+        let persy = recover.finalize().expect("open correctly");
+        let mut tx = persy.begin().expect("error on transaction begin");
+        for _ in 0..20 {
+            let val = String::from("aaa").into_bytes();
+            tx.insert("def", &val).expect("error on insert value");
+        }
+        let fin = tx.prepare().expect("error on commit prepare");
+        fin.commit().expect("error on commit");
+        let counter = persy.scan("def").expect("read persistent records ").count();
+        assert_eq!(ite * 20, counter);
+    }
+}
+
+#[test]
+pub fn test_open_close_restore_multiple_tx() {
+    let file = Builder::new()
+        .prefix("open_restore_close_multiple_tx")
+        .suffix(".persy")
+        .tempfile()
+        .expect("expect temp file creation");
+    Persy::create_from_file(file.reopen().unwrap()).unwrap();
+    {
+        let persy = Persy::open_from_file(file.reopen().unwrap(), Config::new()).unwrap();
+        let mut tx = persy.begin().expect("error on transaction begin");
+        tx.create_segment("def").expect("error on segment creation");
+        let fin = tx.prepare().expect("error on commit prepare");
+        fin.commit().expect("error on commit");
+    }
+    for ite in 1..20 {
+        let recover = OpenOptions::new().recover_file(file.reopen().unwrap()).unwrap();
+        assert!(recover.list_transactions().len() <= 60);
+        let persy = recover.finalize().expect("open correctly");
+        for _ in 0..80 {
+            let mut tx = persy.begin().expect("error on transaction begin");
+            let val = String::from("aaa").into_bytes();
+            tx.insert("def", &val).expect("error on insert value");
+            let fin = tx.prepare().expect("error on commit prepare");
+            fin.commit().expect("error on commit");
+        }
+        let counter = persy.scan("def").expect("read persistent records ").count();
+        assert_eq!(ite * 80, counter);
+    }
+}
+
 #[cfg(feature = "background_ops")]
 #[test]
 pub fn test_background_sync() {
@@ -478,3 +541,141 @@ pub fn test_background_sync() {
         assert!(read_after.is_some());
     });
 }
+
+#[test]
+fn test_operations_no_cache() {
+    let mut config = Config::new();
+    config.change_cache_size(0);
+    create_and_drop_with_config("operations_no_cache", config, |persy| {
+        let rec_data: String = "something".into();
+        let bytes = rec_data.into_bytes();
+        let other_rec_data: String = "other".into();
+        let other_bytes = other_rec_data.into_bytes();
+
+        let mut tx = persy.begin().unwrap();
+        tx.create_segment("test").unwrap();
+        tx.create_index::<u8, u8>("test", ValueMode::Cluster).unwrap();
+        let id_to_update = tx.insert("test", &bytes).unwrap();
+        tx.prepare().unwrap().commit().unwrap();
+
+        let mut tx = persy.begin().unwrap();
+        let id = tx.insert("test", &bytes).unwrap();
+        tx.put::<u8, u8>("test", 10, 20).unwrap();
+        tx.update("test", &id_to_update, &other_bytes).unwrap();
+        tx.prepare().unwrap().commit().unwrap();
+
+        let read_after = persy.read("test", &id).unwrap();
+        assert!(read_after.is_some());
+        let read_after = persy.read("test", &id_to_update).unwrap();
+        assert_eq!(read_after, Some(other_bytes));
+        let mut read_after = persy.get::<u8, u8>("test", &10).unwrap();
+        assert!(read_after.next().is_some());
+
+        let mut tx = persy.begin().unwrap();
+        tx.delete("test", &id).unwrap();
+        tx.delete("test", &id_to_update).unwrap();
+        tx.remove::<u8, u8>("test", 10, None).unwrap();
+        tx.prepare().unwrap().commit().unwrap();
+
+        let read_after = persy.read("test", &id).unwrap();
+        assert!(read_after.is_none());
+        let mut read_after = persy.get::<u8, u8>("test", &10).unwrap();
+        assert!(read_after.next().is_none());
+
+        let mut tx = persy.begin().unwrap();
+        tx.drop_segment("test").unwrap();
+        tx.drop_index("test").unwrap();
+        tx.prepare().unwrap().commit().unwrap();
+    });
+}
+
+#[test]
+pub fn test_not_overgrow_reopen() {
+    let file = Builder::new()
+        .prefix("not_overgrow_reopen")
+        .suffix(".persy")
+        .tempfile()
+        .expect("expect temp file creation");
+    Persy::create_from_file(file.reopen().expect("reopen")).expect(&format!("file '{:?}' do not exist", file));
+    do_ops(&file, |persy| {
+        let mut tx = persy.begin().expect("transaction started");
+        tx.create_segment("def").expect("create segment work");
+        tx.create_index::<u32, u32>("def_index", ValueMode::Cluster)
+            .expect("create index works");
+        tx.prepare().expect("prepare works").commit().expect("commit works");
+        insert_ops(persy);
+    });
+    let size = file.as_file().metadata().expect("error on metadata").len();
+    // Allow 30% margin of overgrow
+    let max = size + size / 3;
+    for i in 0..10 {
+        do_ops(&file, delete_ops);
+        let size = file.as_file().metadata().expect("error on metadata").len();
+        assert!(size < max, " current size: {} max: {} iter:{}", size, max, i);
+        do_ops(&file, insert_ops);
+        let size = file.as_file().metadata().expect("error on metadata").len();
+        assert!(size < max, " current size: {} max: {} iter:{}", size, max, i);
+    }
+}
+
+fn insert_ops(persy: &Persy) {
+    for v in 0..10 {
+        let mut tx = persy.begin().expect("error on transaction begin");
+        let val = String::from("aaa").into_bytes();
+        tx.insert("def", &val).expect("error on insert value");
+        tx.put::<u32, u32>("def_index", v, v).expect("error on index");
+        let fin = tx.prepare().expect("error on commit prepare");
+        fin.commit().expect("error on commit");
+    }
+}
+fn delete_ops(persy: &Persy) {
+    let cursor = persy.scan("def").expect("error on scan");
+    for (id, _) in cursor {
+        let mut tx = persy.begin().expect("error on transaction begin");
+        tx.delete("def", &id).expect("error on insert value");
+        let fin = tx.prepare().expect("error on commit prepare");
+        fin.commit().expect("error on commit");
+    }
+    for v in 0..10 {
+        let mut tx = persy.begin().expect("error on transaction begin");
+        tx.remove::<u32, u32>("def_index", v, Some(v)).expect("error on index");
+        let fin = tx.prepare().expect("error on commit prepare");
+        fin.commit().expect("error on commit");
+    }
+}
+
+pub fn do_ops<F>(file: &NamedTempFile, test: F)
+where
+    F: FnOnce(&Persy),
+{
+    let persy = Persy::open_from_file(file.reopen().expect("reopen"), Config::new()).unwrap();
+    test(&persy);
+}
+
+#[test]
+pub fn test_not_overgrow() {
+    let file = Builder::new()
+        .prefix("not_overgrow")
+        .suffix(".persy")
+        .tempfile()
+        .expect("expect temp file creation");
+    Persy::create_from_file(file.reopen().expect("reopen")).expect(&format!("file '{:?}' do not exist", file));
+    let persy = Persy::open_from_file(file.reopen().expect("reopen"), Config::new()).unwrap();
+    let mut tx = persy.begin().expect("transaction started");
+    tx.create_segment("def").expect("create segment work");
+    tx.create_index::<u32, u32>("def_index", ValueMode::Cluster)
+        .expect("create index works");
+    tx.prepare().expect("prepare works").commit().expect("commit works");
+    insert_ops(&persy);
+    let size = file.as_file().metadata().expect("error on metadata").len();
+    // Allow 30% margin of overgrow
+    let max = size + size / 3;
+    for i in 0..10 {
+        delete_ops(&persy);
+        let size = file.as_file().metadata().expect("error on metadata").len();
+        assert!(size < max, " current size: {} max: {} iter:{}", size, max, i);
+        insert_ops(&persy);
+        let size = file.as_file().metadata().expect("error on metadata").len();
+        assert!(size < max, " current size: {} max: {} iter:{}", size, max, i);
+    }
+}
